#!/bin/bash

# Copyright xmuspeech (Author: Wenxuan Hu, 2021-07-20)
# Corresponding email: olr_challenge@163.com
# Please refer to http://olr.cslt.org for more info

. ./path.sh || exit 1;
. ./cmd.sh || exit 1;

train_dir=$1/$2
dir_train=${train_dir}/OLR2016-OLR2017/train
dir_olr_20dialect=${train_dir}/OLR2020/dialect
dir_olr_20test=${train_dir}/OLR2020/test

# data directory check
if [ ! -d ${train_dir} ] ; then
    echo "Error: train dir"
    exit 1;
fi

echo Preparing dataset
local_dir=data/local
mkdir -p ${local_dir}

#find wav audio file for train
cp ${dir_train}/train/utt2spk $local_dir
cp ${dir_train}/train/wav.scp $local_dir
cp ${dir_train}/train/text ${local_dir}/transcripts.txt

train_olr10="ct-cn id-id ja-jp ko-kr ru-ru Tibet Uyghu Kazak vi-vn zh-cn"
find ${dir_train} -iname "*.wav" | awk '{print $0}' > ${local_dir}/wav_olr10.flist
for rtask in ${train_olr10}; do
    dt_dir=data/dt_${rtask}
    tr_dir=data/tr_${rtask}
    mkdir -p ${dt_dir}
    mkdir -p ${tr_dir}

    grep -i "${rtask}" ${local_dir}/wav_olr10.flist | sort -u > ${tr_dir}/wav_all.flist || exit 1;
    grep -i "${rtask}" ${local_dir}/transcripts.txt | sort -u > ${tr_dir}/text_all || exit 1;
    head -n 300 ${tr_dir}/wav_all.flist > ${dt_dir}/wav.flist 
    sed '1,300d' ${tr_dir}/wav_all.flist > ${tr_dir}/wav.flist 
	
    for dir in ${dt_dir} ${tr_dir}; do
      sed -e 's/\.wav//' ${dir}/wav.flist | awk -F '/' '{print $NF}' | sort -u > ${dir}/utt.org || exit 1
      paste -d '    ' ${dir}/utt.org ${dir}/wav.flist > ${dir}/wav.scp_all || exit 1
      utils/filter_scp.pl -f 1 ${dir}/utt.org ${tr_dir}/text_all | sort -u -k1 > ${dir}/text
      awk '{print $1}' ${dir}/text > ${dir}/utt.list
      utils/filter_scp.pl -f 1 ${dir}/utt.list ${dir}/wav.scp_all | sort -u > ${dir}/wav.scp
      utils/filter_scp.pl -f 1 ${dir}/utt.list ${local_dir}/utt2spk | sort -u > ${dir}/utt2spk
      utils/utt2spk_to_spk2utt.pl ${dir}/utt2spk > ${dir}/spk2utt

      n=`cat ${dir}/wav.scp | wc -l`
      echo ${dir} ${n}
    done
done

####
olr_20dialect="Minnan Sichuan Shanghai"
for rtask in ${olr_20dialect};do
    rtask_dir=${dir_olr_20dialect}/${rtask}
    local_rtask_dir=${local_dir}/${rtask}
    mkdir -p ${local_rtask_dir}
    find ${rtask_dir}/DATA/SCRIPT/ -type f -exec cat {} \;> ${local_rtask_dir}/pre_text
    sed -i 's/\xEF\xBB\xBF//' ${local_rtask_dir}/pre_text
    sed -n '1~2p' ${local_rtask_dir}/pre_text  > ${local_rtask_dir}/1.txt
    sed -n '2~2p' ${local_rtask_dir}/pre_text  > ${local_rtask_dir}/2.txt
    awk '{print $1}' ${local_rtask_dir}/1.txt > ${local_rtask_dir}/utt.list
    paste -d'   ' ${local_rtask_dir}/utt.list  ${local_rtask_dir}/2.txt > ${local_rtask_dir}/text.org
    awk '{print "'$rtask'-"$0}' ${local_rtask_dir}/text.org | sort -u > ${local_rtask_dir}/text

    find ${rtask_dir} -iname "*.WAV" | awk '{print $0}' > ${local_rtask_dir}/wav_org.flist
    n=`cat ${local_rtask_dir}/wav_org.flist | wc -l`
    echo find ${rtask} $n

    sed -e 's/\.WAV//' ${local_rtask_dir}/wav_org.flist | awk -F '/' '{print "'$rtask'-"$NF}'  > ${local_rtask_dir}/utt.list || exit 1
    paste -d'   ' ${local_rtask_dir}/utt.list ${local_rtask_dir}/wav_org.flist > ${local_rtask_dir}/wav.scp_all || exit 1


    awk '{print $1 " '$rtask'"}' ${local_rtask_dir}/utt.list > ${local_rtask_dir}/utt2spk.org
    awk '{print $1}' ${local_rtask_dir}/text > ${local_rtask_dir}/utt.list
    utils/filter_scp.pl -f 1  ${local_rtask_dir}/utt.list ${local_rtask_dir}/wav.scp_all | sort -u > ${local_rtask_dir}/wav.scp
    utils/filter_scp.pl -f 1 ${local_rtask_dir}/utt.list ${local_rtask_dir}/utt2spk.org | sort -u > ${local_rtask_dir}/utt2spk
    utils/utt2spk_to_spk2utt.pl ${local_rtask_dir}/utt2spk > ${local_rtask_dir}/spk2utt

    mkdir -p data/tr_${rtask}
    cp ${local_rtask_dir}/text  data/tr_${rtask}
    cp ${local_rtask_dir}/wav.scp  data/tr_${rtask}
    cp ${local_rtask_dir}/utt2spk  data/tr_${rtask}
    cp ${local_rtask_dir}/spk2utt  data/tr_${rtask}
done

for rtask in ${olr_20dialect}; do
    source_dir=data/tr_${rtask}
    target_dir=data/dt_${rtask}
    mkdir -p ${target_dir}

    head -n 300 ${source_dir}/wav.scp > ${target_dir}/wav.scp
    sed -i.bak '1,300d' ${source_dir}/wav.scp
    
    mv ${source_dir}/text ${source_dir}/text.org
    mv ${source_dir}/utt2spk ${source_dir}/utt2spk.org

    for dir in ${source_dir} ; do
        awk '{print $1}' ${dir}/wav.scp > ${dir}/utt.list
        utils/filter_scp.pl -f 1 ${dir}/utt.list ${source_dir}/text.org | sort -u > ${dir}/text
        utils/filter_scp.pl -f 1 ${dir}/utt.list ${source_dir}/utt2spk.org | sort -u > ${dir}/utt2spk
        utils/utt2spk_to_spk2utt.pl ${dir}/utt2spk > ${dir}/spk2utt
        n=`cat ${dir}/wav.scp | wc -l`
        echo make ${dir} $n
    done
    for dir in ${target_dir}; do
        awk '{print $1}' ${dir}/wav.scp > ${dir}/utt.list
        utils/filter_scp.pl -f 1 ${dir}/utt.list data/tr_${rtask}/text.org | sort -u > ${dir}/text
        awk '{print $1 " '$rtask'"}' ${dir}/utt.list > ${dir}/utt2spk.org
        utils/filter_scp.pl -f 1 ${dir}/utt.list ${dir}/utt2spk.org | sort -u > ${dir}/utt2spk
        utils/utt2spk_to_spk2utt.pl ${dir}/utt2spk > ${dir}/spk2utt
        n=`cat ${dir}/wav.scp | wc -l`
        echo make ${dir} $n
    done
done

###
tr_eval2020="ct-cn id-id ja-jp ko-kr ru-ru vi-vn zh-cn Minnan Sichuan Shanghai"
for lang in ${tr_eval2020};do
    lang_dir=${dir_olr_20test}/${lang}
    label=`ls ${lang_dir}`
    echo $label
   
    for tag in ${label};do
        rtask=${lang}_${tag}
        rtask_dir=${lang_dir}/${tag}

        local_rtask_dir=${local_dir}/${rtask}
        mkdir -p ${local_rtask_dir}
        find ${rtask_dir}/DATA/SCRIPT/ -type f -exec cat {} \;> ${local_rtask_dir}/pre_text
    
        sed -i 's/\xEF\xBB\xBF//' ${local_rtask_dir}/pre_text
        sed -n '1~2p' ${local_rtask_dir}/pre_text  > ${local_rtask_dir}/1.txt
        sed -n '2~2p' ${local_rtask_dir}/pre_text  > ${local_rtask_dir}/2.txt
        awk '{print $1}' ${local_rtask_dir}/1.txt > ${local_rtask_dir}/utt.list
        paste -d'   ' ${local_rtask_dir}/utt.list  ${local_rtask_dir}/2.txt > ${local_rtask_dir}/text.org
        awk '{print "'$rtask'-"$0}' ${local_rtask_dir}/text.org | sort -u > ${local_rtask_dir}/text

        find ${rtask_dir} -iname "*.WAV" | awk '{print $0}' > ${local_rtask_dir}/wav_org.flist

        sed -e 's/\.WAV//' ${local_rtask_dir}/wav_org.flist | awk -F '/' '{print "'$rtask'-"$NF}'  > ${local_rtask_dir}/utt.list || exit 1
        paste -d'   ' ${local_rtask_dir}/utt.list ${local_rtask_dir}/wav_org.flist > ${local_rtask_dir}/wav.scp_all || exit 1


        awk '{print $1 " '$rtask'"}' ${local_rtask_dir}/utt.list > ${local_rtask_dir}/utt2spk.org
        awk '{print $1 " '$lang'"}' ${local_rtask_dir}/utt.list > ${local_rtask_dir}/utt2lang.org


        awk '{print $1}' ${local_rtask_dir}/text > ${local_rtask_dir}/utt.list
        utils/filter_scp.pl -f 1  ${local_rtask_dir}/utt.list ${local_rtask_dir}/wav.scp_all | sort -u > ${local_rtask_dir}/wav.scp
        utils/filter_scp.pl -f 1 ${local_rtask_dir}/utt.list ${local_rtask_dir}/utt2spk.org | sort -u > ${local_rtask_dir}/utt2spk
        utils/filter_scp.pl -f 1 ${local_rtask_dir}/utt.list ${local_rtask_dir}/utt2lang.org | sort -u > ${local_rtask_dir}/utt2lang
        utils/utt2spk_to_spk2utt.pl ${local_rtask_dir}/utt2spk > ${local_rtask_dir}/spk2utt

        mkdir -p data/tr_${rtask}
        cp ${local_rtask_dir}/text  data/tr_${rtask}
        cp ${local_rtask_dir}/wav.scp  data/tr_${rtask}
        cp ${local_rtask_dir}/utt2spk  data/tr_${rtask}
        cp ${local_rtask_dir}/utt2lang  data/tr_${rtask}
        cp ${local_rtask_dir}/spk2utt  data/tr_${rtask}

        n=`cat data/tr_${rtask}/wav.scp | wc -l`
        echo make ${rtask} $n
    done
done


mkdir -p data/dev
mkdir -p data/$2

utils/combine_data.sh data/dev data/dt_*
utils/combine_data.sh data/$2 data/tr_*